<?php
namespace Teknisa\COZ\Controllers;

use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;

class Coz01000ConsumoPlanejado extends \Zeedhi\Framework\Controller\Report {

	protected $crudService;
	protected $util;
	protected $dataSourceManager;
	protected $novoCodigo;
	protected $entityManager;
	protected $birtStrategy;

	public function __construct(ReportService $reportService, $crudService, $util, $novoCodigo, $entityManager, $birtStrategy) {
		$this->crudService    	= $crudService;
		$this->util             = $util;
		$this->novoCodigo    	= $novoCodigo;
		$this->entityManager    = $entityManager;
		$this->birtStrategy     = $birtStrategy;
		
		parent::__construct($reportService);
    }
	
	protected $reportMapping = array (
      'cus01000' => 
        array (
            'strategy' => 'BIRT',
            'parameterMapping' => 
            array (
                'DTINICIAL'     => 'P_DTINI',
                'DTFINAL'       => 'P_DTFIN',
                'CDCENTRAL'     => 'P_CDCENTRAL',
                'NRLISTA'       => 'P_NRLISTA',
                'NRCONSPLAN'    => 'P_NRCONSPLAN',
                'ORDENA'        => 'P_ORDENA',
                'CDPICTPROD'    => 'P_CDPICTPROD',
                'CDARVPRODINI'  => 'P_CDARVPRODINI',
                'CDARVPRODFIN'  => 'P_CDARVPRODFIN',
                'NRORG'         => 'P_NRORG'
            )
        ),
        'cus01001' => 
        array (
            'strategy' => 'BIRT',
            'parameterMapping' => 
            array (
                'DTINICIAL'     => 'P_DTINI',
                'DTFINAL'       => 'P_DTFIN',
                'CDCENTRAL'     => 'P_CDCENTRAL',
                'NRLISTA'       => 'P_NRLISTA',
                'NRCONSPLAN'    => 'P_NRCONSPLAN',
                'ORDENA'        => 'P_ORDENA',
                'CDPICTPROD'    => 'P_CDPICTPROD',
                'CDARVPRODINI'  => 'P_CDARVPRODINI',
                'CDARVPRODFIN'  => 'P_CDARVPRODFIN',
                'NRORG'         => 'P_NRORG'
            )
        ),
        'cus01002' => 
        array (
            'strategy' => 'BIRT',
            'parameterMapping' => 
            array (
                'DTINICIAL'     => 'P_DTINI',
                'DTFINAL'       => 'P_DTFIN',
                'CDCENTRAL'     => 'P_CDCENTRAL',
                'NRLISTA'       => 'P_NRLISTA',
                'NRCONSPLAN'    => 'P_NRCONSPLAN',
                'ORDENA'        => 'P_ORDENA',
                'CDPICTPROD'    => 'P_CDPICTPROD',
                'CDARVPRODINI'  => 'P_CDARVPRODINI',
                'CDARVPRODFIN'  => 'P_CDARVPRODFIN',
                'NRORG'         => 'P_NRORG'
            )
        )
    );
	
	public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response){
	    try{
	        
	        $row = $request->getRow();

    	    if(empty($row['NMSERVFILI'])) $row['NMSERVFILI'] = $this->filtraServicos($row);
	    	
    	    $row['NRLISTA'] = $this->gerarListaServ($row['NMSERVFILI'], $row['NRORG']);
    
    	    $nrListaConsplan = $this->gerarLista($row['NMSERVFILI'], $row['NRORG']);

    	    $cdprodini = empty($row['CDARVPRODINI']) ? '0' : $row['CDARVPRODINI'];
            if(empty($row['CDARVPRODFIN'])) $cdprodfin = 'Z';
            else $cdprodfin = str_pad($row['CDARVPRODFIN'], 10, "Z", STR_PAD_RIGHT);
            
            $row['CDARVPRODINI'] = $cdprodini;
            $row['CDARVPRODFIN'] = $cdprodfin;
            
    	    $row['NRCONSPLAN'] = $this->gerarNrConsplan($nrListaConsplan, $row);
    	    $this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));
	    }catch(\Exception $e){
	        $response->setError(new Error($e->getMessage(), 500));
	    }
	}
	
	public function filtraServicos($row){
		 $conditions = [
			 	array(
                    "columnName" => "NRORG",
                    "operator"   => '=',
                    "value"      =>  $this->crudService->getEnvironment()->getNrOrg()
                ),
                array(
                    "columnName" => 'CDCENTRAL',
                    "operator"   => '=',
                    "value"      => $row['CDCENTRAL']
                ),
                array(
                    "columnName" => 'CDGRUPFILI',
                    "operator"   => '=',
                    "value"      => $row['CDGRUPFILI']  === '' ? 'T' : $row['CDGRUPFILI']
                ),
                array(
                    "columnName" => 'CDGRUPSERV',
                    "operator"   => '=',
                    "value"      => $row['NMGRUPSERV'] === '' || sizeof($row['NMGRUPSERV']) == 0 ? 'T' : join('_',$row['NMGRUPSERV'])   
                ),
                array(
                    "columnName" => 'CDFILIAGREGA',
                    "operator"   => '=',
                    "value"      => $row['NMFILIAL'] === '' || sizeof($row['NMFILIAL']) == 0 ? 'T' : join('_',$row['NMFILIAL'])
                ),
                
            ];
         $dataset = $this->crudService->getFilteredDataSet("servico_por_filiais", $conditions);       
 		 $servicos = $dataset->getRows();
 		 
 		 return $servicos;
	}
	
	public function gerarListaServ($servicos, $nrorg) {
     try {   
            
        	$nrlista = $this->novoCodigo->getCodeByTableName('LISTA_SERVICO', $nrorg);
        	$sql = "INSERT INTO LISTA_SERVICO (NRLISTA, CDFILIAL, CDSERVFILI, NMSERVICO, CDGRUPSERV)
        	                    VALUES (:NRLISTA, :CDFILIAL, :CDSERVFILI, :NMSERVICO, :CDGRUPSERV)";
            
        	foreach ($servicos as $servico) {
                $stmt = $this->entityManager->getConnection()->prepare($sql);
                $stmt->execute(
                    array(
                        "NRLISTA"    => $nrlista,
                        "CDFILIAL"   => $servico['CDFILIAL'],
                        "CDSERVFILI" => $servico['CDSERVFILI'],
                        "NMSERVICO"  => $servico['NMSERVICO'],
                        "CDGRUPSERV" => empty($servico['CDGRUPSERV']) ? null : $servico['CDGRUPSERV']
                    )
                );
            }
            return $nrlista;
        } catch (\Exception $e) {
            throw $e;
        }
    }
	
	public function gerarLista($servicos, $nrorg) {
     try {  
        	$nrlista = $this->novoCodigo->getCodeByTableName('LISTA', $nrorg);
        	$sql = "INSERT INTO LISTA (NRLISTA, CDLISTA)
        	                    VALUES (:NRLISTA, :CDLISTA)";
            
        	foreach ($servicos as $servico) {
                $stmt = $this->entityManager->getConnection()->prepare($sql);
                $stmt->execute(
                    array(
                        "NRLISTA" => $nrlista,
                        "CDLISTA" => $servico['CDFILIAL'].$servico['CDSERVFILI']
                    )
                );
            }
            return $nrlista;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
	public function gerarNrConsplan($nrlista, $row) {
		try {
		    $procedure = new StoredProcedure($this->entityManager->getConnection(), 'CONSPLAN_GERAL');
            $procedure->addParam(new Param('NRLISTA', Param::PARAM_INPUT, $nrlista, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('DTINICIAL', Param::PARAM_INPUT, $row['DTINICIAL'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('DTFINAL', Param::PARAM_INPUT, $row['DTFINAL'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('IDPRODUTOS', Param::PARAM_INPUT, 'CFP', Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('IDSEPPRATO', Param::PARAM_INPUT, 'P', Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('PEACRESCIMO', Param::PARAM_INPUT, 0, Param::PARAM_TYPE_INT));
            $procedure->addParam(new Param('IDARREDOND', Param::PARAM_INPUT, 'N', Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('CDPRODUTOINI', Param::PARAM_INPUT, '0', Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('CDPRODUTOFIN', Param::PARAM_INPUT, 'Z', Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('CDOPERADOR', Param::PARAM_INPUT, $this->crudService->getEnvironment()->getCdOperador(), Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('NRCONSPLANTH', Param::PARAM_INPUT, 'GERANOVCOD', Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('NRCONSPLAN', Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 10));
            $nrconsplan = $procedure->execute()['NRCONSPLAN'];
            return $nrconsplan;
		} catch (\Exception $e) {
			throw $e;
    	}
	}
	
	public function excluirParametros(DTO\Request\Row $request, DTO\Response $response){
	    $row = $request->getRow();
	   if(isset($row["NRLISTA"])){
	        $this->util->deleteLista($row["NRLISTA"]);
    	    $connection = $this->entityManager->getConnection();
            $query="DELETE FROM CONSPLAN WHERE NRORG = :NRORG AND NRCONSPLAN = :NRCONSPLAN";
            $connection->executeQuery($query, array("NRORG" => $row['NRORG'], "NRCONSPLAN" => $row['NRCONSPLAN']));
	   }
	   
	}
	
	public function relatorioCustomizavel(DTO\Request\Row $request, DTO\Response $response) {
	    $row = $request->getRow();
	    
	    if(empty($row['NMSERVFILI'])) $row['NMSERVFILI'] = $this->filtraServicos($row);
	    $row['NRLISTA'] = $this->gerarListaServ($row['NMSERVFILI'], $row['NRORG']);
    	$nrListaConsplan = $this->gerarLista($row['NMSERVFILI'], $row['NRORG']);
    	$row['NRCONSPLAN'] = $this->gerarNrConsplan($nrListaConsplan, $row);
    	
    	$cdprodini = empty($row['CDARVPRODINI']) ? '0' : $row['CDARVPRODINI'];
        if(empty($row['CDARVPRODFIN'])) $cdprodfin = 'Z';
        else $cdprodfin = str_pad($row['CDARVPRODFIN'], 10, "Z", STR_PAD_RIGHT);
        
        $row['CDARVPRODINI'] = $cdprodini;
        $row['CDARVPRODFIN'] = $cdprodfin;
	    
	    $sql = "SELECT
                    F.CDFILIAL AS CDCENTRAL,
                    F.NMFILIAL AS NMCENTRAL,
                    :P_DTINI        AS DTINICIAL,
                    :P_DTFIN        AS DTFINAL,
                	S.CDGRUPSERV AS CODIGOGRUPO, S.NMGRUPSERV AS GRUPOSERVICO,
                    TO_CHAR(C.DT,'DD/MM/YYYY') AS DATA,
                    TO_CHAR(C.DT,'Day') AS DIA,
                    C.CDPRODPRODUCAO,
                    PROD.NMPRODUTO  AS PRODUTOACABADO,
                    NVL(COMENSAISDIA.QTCOMENSAIS, 0) AS COMENSAIS,
                    P.CDARVPROD AS CODIGO,
                	DECODE(
                	   C.IDTPREQPROD,
                	   'D',
                	   '* ',
                	   '  '
                	)|| P.NMPRODUTO AS PRODUTO,
                    SUM(C.QTPROD_PLA) AS QTDE,
                    P.SGUNIDADE AS UN,
                    DECODE(NVL(COMENSAISDIA.QTCOMENSAIS,0), 0, 0, SUM(C.QTPROD_PLA) / COMENSAISDIA.QTCOMENSAIS) AS PERCAPITAPLA,
                    F.NRORG,
                    C.IDTPREQPROD
                FROM
                ( 
                		SELECT
                             CA.DTCARDAPIO AS DT,
                             SUM(CA.QTCOMESTICAR) AS QTCOMENSAIS,
                			 L.CDGRUPSERV
                          FROM
                             CARDADIA CA,
                             LISTA_SERVICO L
                          WHERE
                             CA.DTCARDAPIO BETWEEN :P_DTINI AND :P_DTFIN AND 
                             L.NRLISTA = :P_NRLISTA AND 
                             L.CDFILIAL = CA.CDFILIAL AND 
                			 L.CDSERVFILI = CA.CDSERVFILI
                          GROUP BY
                             CA.DTCARDAPIO, L.CDGRUPSERV) COMENSAISDIA,
                    (
                             /*ITENS PLANEJADOS*/
                                   SELECT
                                    CONS.DTCONSPLAN AS DT,
                			        CONS.CDPRODPRODUCAO,
                                    CONS.CDPRODCPLA AS CDPRODUTO,
                                    SUM(CONS.QTCONSPLAN) AS QTPROD_PLA,
                                    0 QTPROD_REAL,
                                    CONS.IDTPREQPROD, CONS.CDGRUPSERV
                                FROM
                                    (
                                    /*ITENS DE PRODUCAO DO PA*/
                                    SELECT      
                                            C.DTCONSPLAN,
                							R.CDPRODPRODUCAO,
                                            C.CDPRODCPLA, 
                                            (C.QTCONSPLAN) AS QTCONSPLAN,
                                            DECODE(R.IDTPREQPROD, 'I', 'P', R.IDTPREQPROD) IDTPREQPROD, L.CDGRUPSERV 
                                        FROM
                                            CONSPLAN C
                                            INNER JOIN TABLE ( RECEITA_UTIL(C.CDFILICPLA,C.CDSERVCPLA,C.CDPRATCONSPL,C.CDFAPRATCPLA,'0','Z',NULL,0,0,'S') ) R
                                                ON R.CDPRODPRODUCAO IS NOT NULL
                                                AND R.IDTPREQPROD <> 'E'
                								AND R.CDPRATO = C.CDPRATCONSPL
                								AND R.CDFILAUXPRAT = C.CDFAPRATCPLA
                                            INNER JOIN LISTA_SERVICO L ON L.NRLISTA = :P_NRLISTA
                												  AND C.CDFILICPLA = L.CDFILIAL AND C.CDSERVCPLA = L.CDSERVFILI
                                        WHERE C.NRCONSPLAN =:P_NRCONSPLAN
                                            AND ( E_AGREGADA(:P_CDCENTRAL,C.CDFILICPLA,C.CDSERVCPLA,C.CDTPPRATCPLA,C.DTCONSPLAN) > 0 )
                                            AND C.IDTPREQPROD <> 'E'
                                            AND R.CDPRODUTO = C.CDPRODCPLA
                							AND R.IDTPREQPROD = C.IDTPREQPROD
                                        GROUP BY
                                            C.DTCONSPLAN, 
                							R.CDPRODPRODUCAO, C.CDPRODCPLA,
                							C.QTCONSPLAN,
                                            R.IDTPREQPROD, L.CDGRUPSERV
                                UNION  
                                /*ITENS DE PRODUCAO PARA FINALIZACAO*/
                                SELECT 
                                            C.DTCONSPLAN,
                							R.CDPRODPRODUCAO,
                                            R.CDPRODUTO,
                                            SUM(C.QTCOMPLPRODCPLA * C.QTCOMESTCPLA) AS QTCONSPLAN,
                                            'D' AS IDTPREQPROD, L.CDGRUPSERV 
                                        FROM
                                            CONSPLAN C
                                            INNER JOIN TABLE ( RECEITA_UTIL(C.CDFILICPLA,C.CDSERVCPLA,C.CDPRATCONSPL,C.CDFAPRATCPLA,'0','Z',NULL,0,0,'S') ) R
                                                ON R.CDPRODPRODUCAO IS NOT NULL
                                                AND R.IDTPREQPROD <> 'E' AND R.IDFINALIZACAO = 'F'
                								AND R.CDPRATO = C.CDPRATCONSPL
                								AND R.CDFILAUXPRAT = C.CDFAPRATCPLA
                                            INNER JOIN LISTA_SERVICO L ON L.NRLISTA = :P_NRLISTA
                												  AND C.CDFILICPLA = L.CDFILIAL AND C.CDSERVCPLA = L.CDSERVFILI
                                        WHERE C.NRCONSPLAN =:P_NRCONSPLAN
                                            AND ( E_AGREGADA(:P_CDCENTRAL,C.CDFILICPLA,C.CDSERVCPLA,C.CDTPPRATCPLA,C.DTCONSPLAN) > 0 )
                                            AND C.IDTPREQPROD <> 'E'
                                            AND NVL(C.QTCOMPLPRODCPLA, 0) > 0
                                            AND R.CDPRODUTO = C.CDPRODCPLA
                							AND R.IDTPREQPROD = C.IDTPREQPROD
                						GROUP BY C.DTCONSPLAN,
                							R.CDPRODPRODUCAO,
                                            R.CDPRODUTO, L.CDGRUPSERV 
                                        UNION ALL
                					/*ITENS DE CONSUMO*/
                						SELECT 
                							C.DTCONSPLAN,
                							' ' AS CDPRODPRODUCAO,
                                            C.CDPRODCPLA AS CDPRODUTO,
                							SUM(C.QTCONSPLAN) AS QTCONSPLAN,
                                            C.IDTPREQPROD, L.CDGRUPSERV
                						FROM
                							CONSPLAN C
                						INNER JOIN LISTA_SERVICO L ON L.NRLISTA = :P_NRLISTA
                												  AND C.CDFILICPLA = L.CDFILIAL AND C.CDSERVCPLA = L.CDSERVFILI
                						WHERE
                							C.NRCONSPLAN =:P_NRCONSPLAN
                							AND C.DTCONSPLAN BETWEEN :P_DTINI AND :P_DTFIN
                							AND ( E_AGREGADA(:P_CDCENTRAL,C.CDFILICPLA,C.CDSERVCPLA,C.CDTPPRATCPLA,C.DTCONSPLAN) > 0 )
                							AND C.IDTPREQPROD <> 'E'
                							AND C.IDCONSPLAN IN ('P','F')
                						GROUP BY
                							C.DTCONSPLAN,
                                            C.CDPRODCPLA,
                                            C.IDTPREQPROD, L.CDGRUPSERV
                                    ) CONS
                                GROUP BY
                                    CONS.DTCONSPLAN,
                			        CONS.CDPRODPRODUCAO,
                                    CONS.CDPRODCPLA,
                                    CONS.IDTPREQPROD, CONS.CDGRUPSERV
                     ) C
                    LEFT JOIN PRODUTO PROD ON PROD.CDPRODUTO = C.CDPRODPRODUCAO
                    INNER JOIN PRODUTO P ON P.CDPRODUTO = C.CDPRODUTO
                    INNER JOIN FILIAL F ON F.CDFILIAL = :P_CDCENTRAL
                    INNER JOIN GRUPSERV S ON S.CDGRUPSERV = C.CDGRUPSERV
                WHERE P.CDARVPROD BETWEEN :P_PRODINI AND :P_PRODFIN
                    AND COMENSAISDIA.DT = C.DT AND COMENSAISDIA.CDGRUPSERV = C.CDGRUPSERV
                GROUP BY
                	F.CDFILIAL,
                    F.NMFILIAL,
                	S.CDGRUPSERV, S.NMGRUPSERV,
                    C.DT,
                    C.CDPRODPRODUCAO,
                    PROD.NMPRODUTO, COMENSAISDIA.QTCOMENSAIS,
                    P.CDARVPROD,
                	P.NMPRODUTO,
                    P.SGUNIDADE,
                    F.NRORG,
                    C.IDTPREQPROD
                ORDER BY S.CDGRUPSERV, S.NMGRUPSERV,
                    C.DT,
                	DECODE(:P_ORDENA, 'C', C.CDPRODPRODUCAO, PROD.NMPRODUTO),
                    DECODE(:P_ORDENA, 'C', P.CDARVPROD, P.NMPRODUTO)";

	    $params = array (
	        "P_CDCENTRAL"  => $row["CDCENTRAL"],
	        "P_DTINI"      => $row["DTINICIAL"],
	        "P_DTFIN"      => $row["DTFINAL"],
	        "P_NRLISTA"    => $row["NRLISTA"],
	        "P_NRCONSPLAN" => $row["NRCONSPLAN"],
	        "P_PRODINI"    => $row["CDARVPRODINI"],
	        "P_PRODFIN"    => $row["CDARVPRODFIN"],
	        "P_ORDENA"     => $row["ORDENA"]
	   );
	   
	    $data = $this->entityManager->getConnection()->fetchAll($sql, $params);
        $response->addDataset(new DataSet('relCustomizavel', array($data)));
	}
}